%%  Matlab file: pdf_SETAR211_Cauchy_2versions.m  
%%
%%  Coded by: JDG
%%
%%  Purpose: Compute EXACT pdf
%%  of SETAR(2;1,1) process with parameter alpha = -beta 
%%  and Cauchy(0,1) innovations varepsilon_{t}.
%% 
%%  References: 
%%  Andel, J. and Barton, T. (1986).
%%     A note on the threshold AR(1) model with Cauchy innovations.
%%     Journal of Time Series Analysis, 7, 1--5.
%%
%%  The Andel-Barton density is NOT VALID; see Li and Tong (2020)
%%  Li, D. and Tong, H. (2020).
%%     On an absolute autoregressive model and skew symmetric distributions. 
%%     Statistica, LXXXX, n. 2.
%%
%%  varepsilon_{t} ~ C(0,1) stationary distribution of the process:
%%
%%  Y_{t} = a |Y_{t-1}| + varepsilon_{t}  with |a|<1,
%%
%%  for "a" positive and negative
%%  are given below by "f1" (f2), and "f1_C" (f2_C).
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all; clc;

alpha  = -0.8;
A      = alpha/(1-abs(alpha));

sigma  = 1;
pdf    = zeros(1,1001);
index  = zeros(1,1001);

for i=1:1001    % Andel-Barton density
  u        = -10+(i-1)*0.1;  
  f1       = (2*A)/(pi*pi) * ( ((1+A)*pi) / ( 2*A*( (1+A)^(2) + u*u) ) - ...
             ( u*log(A^(-2)*(1+u*u) ) + ( A^(2) - 1 + u*u )*atan(u) ) ...
             / ( 4*A^(2)*u*u + (1-A^(2) + u*u)^(2) ) ); 
  pdf1(i)  =  f1; % density
  index(i) =  u;
end

alpha  = 0.8;
A      = alpha/(1-abs(alpha)); 

sigma  = 1;
pdf    = zeros(1,1001);
index  = zeros(1,1001);

for i=1:1001   % Andel-Barton density
  u        = -10+(i-1)*0.1;  
  f2       = (2*A)/(pi*pi) * ( ((1+A)*pi) / ( 2*A*( (1+A)^(2) + u*u) ) + ...
             ( u*log(A^(-2)*(1+u*u) ) + ( A^(2) - 1 + u*u )*atan(u) ) ...
             / ( 4*A^(2)*u*u + (1-A^(2) + u*u)^(2) ) ); 
  pdf2(i)  =  f2; % density
  index(i) =  u;
end

figure(1)
plot(index,pdf1)
hold on
plot(index,pdf2)
hold off

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% New density f_1_C and f_2_C, due to Li and Tong (Statistica, 2020) %%%%%

beta = -0.8;  % beta = alpha of SETAR(2;1,1)

B    = beta;  % beta/(1-abs(beta)) % for equivalence with asMA(1,1); abs(beta) < 1

for i=1:1001
  u     = -60+(i-1)*0.1;  
  part1 = B*u*log( (u^2 + 1)/B^2 );
  part2 = B*(u^2 + B^2 -1)*atan(u);
  part3 = abs(B) * pi * (u^2 + B^2 -1)/2;
  part4 = pi * (u^2 - B^2 + 1)/2;
  part0 = 2/( pi^2 * ( u^2 + (B-1)^2 ) * ( u^2 + (B +1)^2 ) );
  
  f1_C      = (part1 + part2 + part3 + part4)*part0;
  pdf1_C(i) = f1_C;
  index1(i) = u;
end  

beta = 0.8;
B    = beta;   %  beta/(1-abs(beta)); % for equivalence with asMA(1,1) process.

for i=1:1001
  u     = -20+(i-1)*0.1;  
  part1 = B*u*log((u^2 + 1)/B^2);
  part2 = B*(u^2 + B^2 -1)*atan(u);
  part3 = abs(B) * pi * (u^2 + B^2 -1)/2;
  part4 = pi*(u^2 - B^2 + 1)/2;
  part0 = 2/( pi^2* ( u^2+(B-1)^2 ) * ( u^2 + (B +1)^2 ) );
  
  f2_C      = (part1+part2+part3+part4)*part0;
  pdf2_C(i) = f2_C;
  index2(i) = u;
end  
  
figure(2)
%  plot(index,pdf1)
%  hold on
%  plot(index,pdf2)
%  hold on
plot(index1,pdf1_C)
hold on
plot(index2,pdf2_C)
hold off


%%%%%%%%%%%%%%%%%%%%% END MAIN %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%