%  File: pdf_SETAR_Laplace.m 
%
%  Coded by: JDG
% 
%  Computes the exact values of the mean, variance, skewness, kurtosis, 
%  and first-lag autocorrelation function of the SETAR(2;1,1) process (54) 
%  with Laplace(0,1) innovations. Results are displayed in Table 3 of 
%  De Gooijer (2022).
%
%  References:
%   Loges, W. (2004).
%     The stationary marginal distribution of a threshold AR(1) process.
%     Journal of Time Series Analysis, 25, 103--125.
%
%   De Gooijer, J.G. (2022). The marginal distribution function of
%     threshold-type processes with central symmetric innovations.
%     Statistics, 56.
%
%   Ranocha, P. (2006).
%     Stationary distribution of time series}. Doctoral thesis, 
%     Charles University, Prague. Available at:
%     \url{https://is.cuni.cz/webapps/zzp/detail/44509/?lang=en}.
%
%      Y_{t} = alpha Y_{t-1} + varepsilon_{t} if Y_{t-1} >= 0
%      Y_{t} = beta Y_{t-1}  + varepsilon_{t} if Y_{t-1} <  0
%
%   where alpha = a  (alpha_p); beta = b (alpha_m)
%   Cases:
%   1): 0< alpha < 1, 0 < beta < 1;   positive parameters
%   2): parameters of opposite sign
%   3): 0< alpha < 1, beta = - alpha; opposite sign with same 
%                                          absolute value
% OUTPUT:
% Case 3
% Parameters: a (Y(t-1) > 0) and  b = - a 
%     0.5000   -0.5000
% Non-central moments: nu_1, nu_2, nu_3, nu_4
%    0.6091    2.6667    4.6992   34.1333
%    Var     skewness  kurtosis  acvf_1_3  rho_1_3
%   2.2956    0.0799    5.3526    0.7242    0.1538
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all; clc;

a      = 0.5;   % Case 1, Case 2 and Case 3 
%b     = 0.85;  % Case 1;

%b     = -0.75; % Case 2
b      = -a;    % Case 3

par    = [a, b]; % parameters

Q      = 1001; 
Umax   = 201;          
pdf1   = zeros(1,Umax);
pdf2   = zeros(1,Umax);
pdf3   = zeros(1,Umax);

index  = zeros(1,Umax);
c1_a   = zeros(1,Q);
c1_b   = zeros(1,Q);
c2_a   = zeros(1,Q);
c2_b   = zeros(1,Q);
c3_a   = zeros(1,Q);
c3_b   = zeros(1,Q);
d1_a   = zeros(1,Q-1);
d1_b   = zeros(1,Q-1);
s_a    = zeros(1,Q-1);
s_b    = zeros(1,Q-1);

d1_a_a = zeros(1,Q);

r_a    = zeros(1,Q-1);
r_b    = zeros(1,Q-1);

c1_a_0 = a^(2)/(a^(2)-1);
c1_b_0 = b^(2)/(b^(2)-1);

c2_a_0 = 1/(2*(1-a));
c2_b_0 = 1/(2*(1-b));

c3_a_0 = 1/(2*(1+a));
c3_b_0 = 1/(2*(1+b));

for q = 1:Q
  c1_a(q) = a^(2*q+2)/(a^(2*q+2)-1);  
  c1_b(q) = b^(2*q+2)/(b^(2*q+2)-1);

  c2_a(q) = 1/( 2*(1-a^(q+1)) );
  c2_b(q) = 1/( 2*(1-b^(q+1)) );

  c3_a(q) = 1/( 2*(1+a^(q+1)) );
  c3_b(q) = 1/( 2*(1+b^(q+1)) );
end

prod_a  = c1_a_0;
d1_a(1) = prod_a;

prod_b  = c1_b_0;
d1_b(1) = prod_b;

% d1_a(q) = \prod_{q=2}^{Q} c1_a(q-1),  p. 110 of Loges (JTSA)

for q = 2:Q
  prod_a  = prod_a * c1_a(q-1);
  d1_a(q) = prod_a;       
  
  prod_b  = prod_b * c1_b(q-1);
  d1_b(q) = prod_b; 
end

% Definition 5, d1(\lambda,0) = 1
% g1_a =  sum_{q=0}^{\infty} d1(a,q)

g1_a = 1 + sum(d1_a);    
g1_b = 1 + sum(d1_b);  

for q = 1:(Q-1)
  c3_d1_a(q) = c3_a(q).*d1_a(q);  
  c3_d1_b(q) = c3_b(q).*d1_b(q);
end

% Above Definition 5 on p. 112 of Loges (JTSA)
% g2(\lambda) = sum^{\infty}_{q=0} c3(\lambda,q) * d1(\lambda,q)

g2_a = c3_a_0 + sum(c3_d1_a);  
g2_b = c3_b_0 + sum(c3_d1_b);

g3_a = g1_a/g2_a;        % p. 112, Definition 5 of Loges (JTSA)
g3_b = g1_b/g2_b;
g_ab = 1/(g3_a + g3_b);  % page 113 top Eqn., Loges (JTSA)

sum_g4_a_p1 = 0;  % p. 117 g4(a,1) = sum^{\infty}_{q=0} d1(a,q)a^{q}
sum_g4_a_p2 = 0;  %        g4(a,2) = sum^{\infty}_{q=0} d1(a,q)a^{2q} 

sum_g4_b_p1 = 0;  %        g4(b,1) = sum^{\infty}_{q=0} d1(b,q)b^{q}
sum_g4_b_p2 = 0;  %        g4(b,2) = sum^{\infty}_{q=0} d1(b,q)b^{2q}

for q=1:Q  
  sum_g4_a_p1 = sum_g4_a_p1 + d1_a(q) * a^(q);    % p = 1
  sum_g4_a_p2 = sum_g4_a_p2 + d1_a(q) * a^(2*q);  % p = 2
  
  sum_g4_b_p1 = sum_g4_b_p1 + d1_b(q) * b^(q);    % p = 1
  sum_g4_b_p2 = sum_g4_b_p2 + d1_b(q) * b^(2*q);  % p = 2
end

% g4_a is a factor "a" different from g4 in Proposition 8, p=2, of Loges (JTSA)
g4_a_p1 = sum_g4_a_p1 + 1;  % d1_a(0) = 1 
g4_a_p2 = sum_g4_a_p2 + 1;  % g4_a_p2 = g4_b_p2 for a = -b

g4_b_p1 = sum_g4_b_p1 + 1;
g4_b_p2 = sum_g4_b_p2 + 1;

% g4(lambda,2p)   = g1(lambda) ( \prod_{j=1} (1-lambda^(2j) )^(-1) p. 117
% g4(lambda,2p-1) = 2 (g1_a/g3_a) ( \prod_{j=1} (1-lambda^(2j-1) )^(-1)
% for 0<= lambda < 1 !!!

% Corollary 3, p. 114, with q = 0 and d1(alpha,0) = 1
% f_{alpha} = [2*g1(alpha)]^(-1) * d1(alpha,0) * exp( -abs(u) )

% Case 1: Parameters of positive sign
if (a >= 0) && (b >= 0)
  disp('Case 1')
 
  for i=1:Umax  % BIG LOOP
    u     = -10+(i-1)*0.1;
    f_a_0 = (2*g1_a)^(-1) * exp(-abs(u));  % Laplace densities, q = 0
    f_b_0 = (2*g1_b)^(-1) * exp(-abs(u));  
  
    r_0 = exp(-u);   % with d_{1}(lambda,0) = 1

    % Corollary 3, with q > 0 
    % f_{alpha} = sum^{\infty}_{q=1} d1(alpha,q) * alpha^{-q} * exp( -abs(u) )

    % Laplace densities q>0
    for q=1:(Q-1)
      s_a(q)  = d1_a(q) * a^(-q) * exp( -a^(-q) * abs(u) ); % Corollary 3
      s_b(q)  = d1_b(q) * b^(-q) * exp( -b^(-q) * abs(u) ); 
   
      r_a(q)  = d1_a(q) * a^(-q) * exp( -a^(-q) * abs(u) ); 
      r_b(q)  = d1_b(q) * b^(-q) * exp( -b^(-q) * abs(u) );   
    end
  
    s_a_tot = f_a_0 + (2*g1_a)^(-1) * nansum(s_a); % Corollary 3, p. 114, total
    s_b_tot = f_b_0 + (2*g1_b)^(-1) * nansum(s_b); % of above terms
    
    r_a_tot = r_0 + nansum(r_a);  % Below Eqn. (23), Loges (JTSA)
    r_b_tot = r_0 + nansum(r_b);

    % Exact invariant density of the SETAR process. Eqn. (24), Loges (JTSA)
    % 0< a, b < 1
    f1       = 2*g_ab *( g3_a * s_a_tot * (u>=0) + g3_b * s_b_tot * (u<0) ); 
    pdf1(i)  =  f1;
    index(i) =  u;
    
   end  % end i=1:Umax     
   
    % Alternative Non-central moments, p. 117
    nu_1_p1 = g_ab *    ( (g3_a/g1_a)*g4_a_p1 - (g3_b/g1_b)*g4_b_p1 );
    nu_1_p2 = 2 * g_ab* ( (g3_a/g1_a)*g4_a_p2 + (g3_b/g1_b)*g4_b_p2 );
       
    %% Non-central Moments    
    nu_1 = 2 * g_ab * 1 *( (1-a)^(-1) - (1-b)^(-1) );                
    nu_2 = g_ab * factorial(2)*( g3_a * ( (1-a^(2)) )^(-1) + ...
                                 g3_b * ( (1-b^(2)) )^(-1) ) ;        
    nu_3 = 2*g_ab * factorial(3)*( ( (1-a)*(1-a^(3)) )^(-1) - ...
                                 ( (1-b)*(1-b^(3)) )^(-1) );                                     
    nu_4 = g_ab * factorial(4)*( g3_a * ( (1-a^(2)) * (1-a^(4)) )^(-1) + ...
                                 g3_b * ( (1-b^(2)) * (1-b^(4)) )^(-1) ) ; 

    Var = nu_2 - nu_1^(2);  % variance
    sd  = sqrt(Var);        % standard deviation

    % ACVF_1 at lag 1, with g4_a as computed above (NOT g4_a in paper) 
    % see Corollary 6.4 (Eq. (46)) of De Gooijer (2022)
    acvf_1_1 = 2* g_ab * ( (a*g3_a*g4_a_p2/g1_a) + (b*g3_b*g4_b_p2/g1_b) );

    % Central moments
    mu3 = nu_3 - 3* nu_2 * nu_1 + 2*(nu_1)^(3);
    mu4 = nu_4 - 4* nu_3 * nu_1 + 6*nu_2*(nu_1)^(2) - 3*(nu_1)^(4);

    % Skewness and kurtosis
    skew = mu3/sd^(3);
    kurt = mu4/sd^(4);
    
    rho_1_1       = ( acvf_1_1 - nu_1^2 )/Var; % ACF at lag 1
    var_skew_kurt = [Var, skew, kurt, acvf_1_1, rho_1_1];
    
    moments = [nu_1, nu_2, nu_3, nu_4];
    disp('Parameters: a (Y(t-1) > 0) and b (Y(t-1) > 0)')
    disp(par)
    disp('Non-central moments: nu_1, nu_2, nu_3, nu_4')
    disp(moments)
        
    disp('Var  skewness  kurtosis  acvf_1_1  rho_1_1')
    disp(var_skew_kurt)    
       
 end  % Case 1) parameters of positive sign ( a >= 0) && ( b >= 0)
 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if (a >= 0 ) && (b <= 0) && (a~= -b)
  % Case 2) parameters of opposite sign
  disp('Case 2') 
     
  c4_ab_0 = b^(2)/( b^(2) - 1 );  % p. 121 Loges (JTSA)
  c5_ab_0 = 1/ ( 2*(1-abs(b)) );  
   
  for q = 1:Q
     c4_ab(q) = a^(2*q)*b^(2) / ( a^(2*q)*b^(2) - 1 ); % p. 121 Loges (JTSA)
     c5_ab(q) = 1/( 2 * ( 1 - a^(q)*abs(b) ) );  
   end 
     
   c4_ab_new = [c4_ab_0, c4_ab];
   c5_ab_new = [c5_ab_0, c5_ab];
      
   prod4_ab = c4_ab_0;   
   prod5_ab = c5_ab_0;
   
   d3_ab(1) = 1;  
   d3_ab(2) = c4_ab_0;
  
   for q = 2:Q  % NOTE: d3_ab = d2_ab in paper De Gooijer (2022) !!!
     prod4_ab   = prod4_ab * c4_ab(q-1); % d3(alpha,beta,q) on p. 122 (Loges), top 
     d3_ab(q+1) = prod4_ab; 
   end 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
   % g5(alpha,beta) = sum^{\infty}_{q=0} d3(alpha,beta,q) on p. 122 of Loges
   % d3(alpha,beta,0) = 1
   g5_ab = sum(d3_ab); 
                          
   % g6(alpha,beta) = sum^{\infty}_{q=0} c5_ab(q) * d3(alpha,beta,q), p. 122 of Loges
   for q = 1:Q
     prod_c5_d3_ab = c5_ab_new(q).*d3_ab(q);  % d3_ab = vd3
     d6_ab(q)      = prod_c5_d3_ab;   
   end  
    
   %  g6(alpha,beta) = sum^{\infty}_{q=0} c5_ab(q) * d3(alpha,beta,q)
   g6_ab = sum(d6_ab);                       
   
   % sum^{\infty}_{q=1} d3_ab(q)*a^{(q-1)*p} used for g7 (p. 122 of Loges)
     for q=1:(Q-1)
       d3_ab_a_1(q) = d3_ab(q+1) * a^((q-1)*1); 
       d3_ab_a_2(q) = d3_ab(q+1) * a^((q-1)*2);
       d3_ab_a_3(q) = d3_ab(q+1) * a^((q-1)*3);
       d3_ab_a_4(q) = d3_ab(q+1) * a^((q-1)*4); 
     end
   
   % g7(alpha,beta) = |beta|^{p} bsum^{\infty}_{q=1} d3(alpha,beta,q) * alpha^{(q-1)p} 
     g7_ab_1 = abs(b)       * sum(d3_ab_a_1);  % p = 1, 1st moment
     g7_ab_2 = (abs(b))^(2) * sum(d3_ab_a_2);  % p = 2, 2nd  moment 
     g7_ab_3 = (abs(b))^(3) * sum(d3_ab_a_3);  
     g7_ab_4 = (abs(b))^(4) * sum(d3_ab_a_4);
   
     g8_ab = g2_a/g6_ab;     % p. 122 of Loges
   
     g9_ab = ( g3_a * g6_ab + g5_ab )^(-1); % p. 123 of Loges
     
     h_ab  = ( g1_a + g5_ab * g8_ab)^(-1); % p. 122 of Loges
  
     % d1_a_a(q)  = sum^{\infty}_{q=1} d1_a(q) * a^{-q} exp(-a^(-q) * u)
     % d3_ab_a    = Eq on top of p. 123 of Loges
     % d3_ab_a(q) = sum^{\infty}_{q=1} d3_ab(q)*a^{1-q} |b|^{-1}
     %              *exp(-a^((1-q)(|b|)^(-1))*u)

     for i=1:Umax  % BIG LOOP
       u   = -10+(i-1)*0.1; %;
    
       % part of r_{1}(x,a) p. 122 of Loges, bottom 
       d1_a_a(1)    = exp(-u);
       for q = 2:Q
         d1_a_a(q)  = d1_a(q-1) * (a)^(1-q) * exp(-(a)^(1-q) * u); 
       end 
                           
       for q = 1:Q   %% part of r_{2}(x,a,b) p. 123 of Loges, top    
         d3_ab_a(q) = d3_ab(q+1) * (a)^(1-q) * (abs(b))^(-1) * ...
                      exp( -(a)^(1-q)*(abs(b))^(-1) * u );  
       end 
              
       %%  Define new funtion sum^{infty}_{q=1} d3_ab(q) * a^{2q-1}
       sum_g10_ab = 0;
       for q = 1:Q
         sum_g10_ab = sum_g10_ab + d3_ab(q) * (a)^(2*q+1);
       end
       g10_ab = sum_g10_ab;
      
       r1_u_a(i)   = nansum(d1_a_a);   
       r2_u_ab(i)  = nansum(d3_ab_a);  
  
       % p. 122 of Loges, bottom
       % r1(u,a)   =  sum^{\infty}_{q=0} d_1(a,q) 
       %                         * a^{-q}*exp( -a^{-q} * u)
       % p. 123 of Loges, top
       % r2(u,a,b) =  sum^{\infty}_{q=0} d_3(a,b,q) 
       %             * alpha^{1-q} * |b|^{-1} * exp( -a^{(1-q)*|b|^{-1} * u )
       
       % invariant marginal density of the SETAR process, Thm. 6 of Loges 
       % 0 < a < 1, b < 0
       if (u < 0)
         f2  = h_ab * ( g8_ab * exp(u));
         ff3 = 0;
       else  
         f2  = h_ab * ( r1_u_a(i) + g8_ab * r2_u_ab(i) ); 
       end
         
       pdf2(i)  = f2; 
       r1_2(i)  = r1_u_a(i);
       index(i) = u; 
     
     end % i=1:Umax  % BIG LOOP
   
     %% Non-central Moments    
     nu_1 =                g9_ab * ( 2* g6_ab * (1-a)^(- 1) - 1 + g7_ab_1 );            
     nu_2 = factorial(2) * g9_ab * ( g3_a * g6_ab * ( (1-a^(2)) )^(-1) + 1 + g7_ab_2 );                       
     nu_3 = factorial(3) * g9_ab * ( 2* g6_ab  * ( (1-a)*(1-a^(3)) )^(-1) - 1 + g7_ab_3 );     
     nu_4 = factorial(4) * g9_ab * ( g3_a * g6_ab * ( (1-a^(2)) * (1-a^(4)) )^(-1) + 1 + g7_ab_4 );   
 
     Var = nu_2 - nu_1^(2);  % variance
     sd  = sqrt(Var);        % standard deviation
     
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
      c4_ab_0 = b^(2)/( b^(2) - 1 );  % p. 121 of Loges
   
      for q = 1:Q
        c4_ab(q) = a^(2*q)*b^(2) / ( a^(2*q)*b^(2) - 1 ); % p. 121 of Loges
      end 
      
      prod4_ab = c4_ab_0;   
   
      d3_ab(1) = c4_ab_0;  
      for q = 2:Q                  % NOTE: d3_ab = d2_ab in paper De Gooijer (2022) 
        prod4_ab  = prod4_ab * c4_ab(q-1); % d3(alpha,beta,q) on p. 122 of Loges, top 
        d3_ab(q)  = prod4_ab;  
      end 
      
      sum_g11_ab = 0;
      for q = 1:Q
         sum_g11_ab = sum_g11_ab + d3_ab(q) * (a)^(-2+3*q);
      end
      g11_ab = sum_g11_ab;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%           
          
     % ACVF_1, b = < 0  part; a = > 0, Corollary 6.6 of De Gooijer (2022) 
      
     acvf_1_2 =   2 * b * h_ab * g8_ab ...
                + 2 * a * h_ab * g4_a_p2 ...
                + 2 * a * h_ab * g8_ab * g7_ab_2; 
     
     %% Central moments
     mu3 = nu_3 - 3* nu_2 * nu_1 + 2*(nu_1)^(3);
     mu4 = nu_4 - 4* nu_3 * nu_1 + 6*nu_2*(nu_1)^(2) - 3*(nu_1)^(4);

     %% Skewness and kurtosis
     skew = mu3/sd^(3);
     kurt = mu4/sd^(4); 
     
     moments = [nu_1, nu_2, nu_3, nu_4];
     disp('Parameters: a (Y(t-1) > 0), (Y(t-1) < 0), and "a" NEQ "-b" ')
     disp(par)
     disp('Non-central moments: nu_1, nu_2, nu_3, nu_4')
     disp(moments)
            
     rho_1_2       = ( acvf_1_2 - nu_1^2 )/Var;
     var_skew_kurt = [Var, skew, kurt, acvf_1_2, rho_1_2];
     disp('Var  skewness  kurtosis  acvf_1_2  rho_1_2')
     disp(var_skew_kurt)     
     
  end % Case 2) if (b~=-a) && (b<= 0) 2) parameters of opposite sign
  
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Case 3) parameters of opposite sign and same absolute value
  if (b == -a)
  % Exact invariant marginal density of the SETAR process 
  % 0 < a < 1, b = -a
  disp('Case 3') 

  for i=1:Umax  % BIG LOOP  
    u     = -10+(i-1)*0.1;  
    
    f_a_0 = (2*g1_a)^(-1) * exp(-abs(u));  % Laplace densities, q=0
    f_b_0 = (2*g1_b)^(-1) * exp(-abs(u));  
  
    r_0 = exp(-u);   % with d_{1}(lambda,0) = 1

    % Corollary 3 of Loges, with q > 0 
    % f_{alpha} = sum^{\infty}_{q=1} d1(alpha,q) * alpha^{-q} * exp( -abs(u) )

    % Laplace densities q>0
      for q=1:(Q-1)  % Also p. 124 of Loges
        s_a(q)  = d1_a(q) * a^(-q) * exp( -a^(-q) * abs(u) ); % Corollary 3 
        r_a(q)  = d1_a(q) * a^(-q) * exp( -a^(-q) * abs(u) );
        % Below Eqn. (23), p. 114, Loges 
      end
  
      s_a_tot = f_a_0 + (2*g1_a)^(-1) * nansum(s_a); % Corollary 3, total
      r_a_tot = r_0 + nansum(r_a); % Below Eqn. (23),p. 114 Loges, approx zero for q=11    
  
      f3 = (g3_a)^(-1) * exp(u) * (u<0) + ...
          ( (g1_a)^(-1) * r_a_tot - (g3_a)^(-1) * exp(-u) ) * (u>=0); 
      
      pdf3(i)     = f3;     
      index(i) = u;
   end  % i=1:Umax  % BIG LOOP            
     
      % Non-central Moments    
      nu_1 = 2*(g3_a)^(-1) * ( (1-a)^(-1) - 1 );                      
      nu_2 =                 factorial(2) * ( (1-a^(2)) )^(-1);       
      nu_3 = 2*(g3_a)^(-1) * factorial(3) * ( ( (1-a)*(1-a^(3)) )^(-1)-1 ); 
      nu_4 =                 factorial(4) * ( (1-a^(2)) * (1-a^(4)) )^(-1) ;

      Var = nu_2 - nu_1^(2);  % variance
      sd  = sqrt(Var);        % standard deviation

      c4_ab_0 = b^(2)/( b^(2) - 1 );  % p. 121 of Loges
   
      for q = 1:Q
        c4_ab(q) = a^(2*q)*b^(2) / ( a^(2*q)*b^(2) - 1 ); % p. 121 of Loges
      end 
      c4_ab    = [c4_ab_0, c4_ab];
      
      prod4_ab = c4_ab_0;   
   
      d3_ab(1) = c4_ab_0;  
      for q = 2:Q    % NOTE: d3_ab = d2_ab in paper !!!
        prod4_ab  = prod4_ab * c4_ab(q); % d3(alpha,beta,q) on p. 122 of Loges, top 
        d3_ab(q)  = prod4_ab;  
      end 
      
      sum_g12_a = 0;
      sum_g13_a = 0;
      sum_g13_b = 0;
      sum_g14_a = 0;
      sum_g14_b = 0;
      d1_a_new = [1, d1_a];
      for q = 1:Q
         sum_g12_a = sum_g12_a  + d3_ab(q) * (a)^(2*q+3); 
         sum_g13_a = sum_g13_a  + d1_a_new(q) .* (a)^(2*q+1); 
      end
      
      g12_a  = sum_g12_a; 
      g13_a  = sum_g13_a;  
      
      % Central moments
      mu3 = nu_3 - 3* nu_2 * nu_1 + 2*(nu_1)^(3);
      mu4 = nu_4 - 4* nu_3 * nu_1 + 6*nu_2*(nu_1)^(2) - 3*(nu_1)^(4);

      % Skewness and kurtosis
      skew = mu3/sd^(3);
      kurt = mu4/sd^(4);

      % ACVF, Corollary 6.8 of De Gooijer (2022) 
      
      acvf_1_3 = (2*a*g4_a_p2/g1_a) - (4*a/g3_a);
      rho_1_3  = ( acvf_1_3 - nu_1^2 )/Var;
      
      moments = [nu_1, nu_2, nu_3, nu_4];
      disp('Parameters: a (Y(t-1) > 0) and  b = - a ')
      disp(par)
      disp('Non-central moments: nu_1, nu_2, nu_3, nu_4')
      disp(moments)
            
      var_skew_kurt = [Var, skew, kurt, acvf_1_3, rho_1_3];
      disp('Var  skewness  kurtosis  acvf_1_3  rho_1_3')
      disp(var_skew_kurt)      
            
end  % Case 3) parameters of opposite sign and same absolute value

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if (0 <= a <= 1) && (0 <= b <=1)
  figure(1)
  plot(index,pdf1)
end

if (a >= 0 ) && (b <= 0) && (a~= -b)
  figure(2)
  pdf2(isnan(pdf2))=[];  % remove NANs
  plot(index,pdf2)
end

if (b == -a)
  figure(3)
  plot(index,pdf3)
end

%%%%%%%%%%%%%%%%%%%%  END MAIN %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
